/* Taille de la serie a simuler */
%LET n=100;

/* Omega : matrice de covariance souhaitee pour le bruit blanc */
%LET omega11=1;
%LET omega12=0.6;
%LET omega22=1;
%LET r=(&omega12/sqrt(&omega11*&omega22));
%LET a=(sqrt(1+&r)+sqrt(1-&r)) ;
%LET b=(sqrt(1+&r)-sqrt(1-&r)) ;

/* Matrice Phi du processus VAR(1) */
%LET phi11=-6.2;
%LET phi12=-5.5;
%LET phi21=6.6;
%LET phi22=5.9;

/* Simulation du bruit blanc en dimension 2 avec la matrice de covariance Omega */
DATA tab;
    DO i=1 TO &n;
	e=RANNOR(-1);
	d=RANNOR(-1);
	f=sqrt(&omega11)*0.5*(&a*e+&b*d);
	g=sqrt(&omega22)*0.5*(&b*e+&a*d);
	DROP e d;
	OUTPUT;
	END;
RUN;

/* Simulation du processus VAR(1) en dimension 2 avec la matrice Phi et la matrice Omega */
DATA var1;
    SET tab;
    RETAIN U 0 V 0;
    X=&phi11*U+&phi12*V+f;
    Y=&phi21*U+&phi22*V+g;
    t=_N_;
    U=X;
    V=Y;
    drop U V;
RUN;

/* Graphique des deux series */
 /* Options pour la presentation des lignes */
SYMBOL1 I=join CI=green;
SYMBOL2 I=join CI=red;

PROC GPLOT DATA=var1;
    PLOT X*t;
    PLOT Y*t;
    PLOT X*t Y*t /overlay;
RUN;
QUIT;

 /* Procedure VARMAX servant a l'estimation de la fonction d'autocovariance */
/* l'option CORRY affiche la fonction d'autocorrelation */

ods OUTPUT CORRYlags=corr;
PROC VARMAX DATA=var1;
    MODEL X Y / PRINT=(CORRY);
RUN;
QUIT;

ODS OUTPUT CLOSE;

PROC PRINT data=corr;
RUN;
QUIT;

/* Pour affichier la correlation croisee rho12(h) */
DATA crosscorrXY (KEEP= rhoxy h);
    SET corr;
    if MOD(_N_,2) = 1 ;
    rhoxy = Y;
    h = LAG;
RUN;

DATA crosscorrYX (KEEP= rhoxy h );
    SET corr;
    if MOD(_N_,2) = 0 ;
    rhoxy = X;
    h = - LAG;
RUN;

PROC SORT data=crosscorryx; 
    BY h;
RUN;
QUIT;

DATA crosscorr;
    SET crosscorryx  crosscorrxy ;
RUN;

PROC GPLOT data=crosscorr;
    PLOT rhoxy*h;
RUN;
QUIT;
