/*** Options debougage ***/
options symbolgen  mprint mlogic spool; 
/*** Option : imprimer un les tableaux intermediaires ***/
%let verbose=0;

/*** Repertoire des donnes ***/
%let CHEMIN=C:\Documents and Settings\lsantoca\Mes documents\My SAS Files\V8\TD4\;

/*** Les actifs a consider ***/
%let actifs=aventis axa bnp danone loreal lvmh sanofi socgen total; 
/*%let actifs=aventis total;*/


%macro nomactifs;

    %let i=1; 
    %do %while(%scan(&actifs,&i) ne );
	%global  act&i;
	%let act&i=%scan(&actifs,&i);
	%put( &&act&i);
	%let i=%eval(&i +1);
	%end;

    %global  noactifs;
    %let noactifs=%eval(&i -1);

    %mend nomactifs;

%nomactifs;


/*** Variable dont on veut estimer la moyenne et la covariance ***/
/** Excess gain **/
%let varconst=Ygrand;
/* Rendements non actualises */
/* %let varconst=ypetit; */


/********* Donnes communes des portfeuilles **********************/
/** Coefficient d'aversion au risque **/
%let A=0.0001;
/** Montant a investir (10000 euros) **/
%let w=10000;  
/** Jour des investissement **/
%let jourinvest='29NOV2002'd;
/** Taux fixe sur no jours **/
%let taux=0.02884;
/* Sur combien de jours */
%let jours=7;
/* Calculs du taux fixe jouranlier */
%let tauxj=%sysevalf(&taux/&jours);
/*****************************************************************/


/*********** MACROS ***********************************************/

/*** Lire les donnees d'un actifs, 
    calculer les "Excess Gain" et les rendements non actualises ***/

%macro lireactif(actif,tauxj);
    %global dernier&actif;
     
    %let FICHIER=&actif..tab;
    FILENAME &actif "&CHEMIN&FICHIER";
    
    DATA &actif (keep=DATE prix ypetit Ygrand  
		     rename=(prix = prix&actif 
				 ypetit = ypetit&actif 
				 Ygrand=Ygrand&actif) );
	INFILE &actif DSD dlm='09'x truncover firstobs=2;
	INPUT DATE:ddmmyy10. NOM$ SICOVAM PREMIER; 
	FORMAT DATE date6.;
	jtaux=&tauxj;
	prix=PREMIER;
	lag=LAG(prix);
	dif=DIF(prix);
	ypetit=dif/lag;
	Ygrand=prix - (1 + LAG(jtaux))*LAG(prix);
    RUN;

    DATA _null_;
	set &actif;
	if(DATE = &jourinvest);
	call symput("dernier"||"&actif",prix&actif);
    RUN;

    
    %mend lireactif;

/*** Macro : calcul de l'allocation optimale des actifs 
   et de la performance de Sharp ***/
 
%macro allocation_performace;
    
    DATA _null_;
	SET means;
	call SYMPUT("mu"||put(_N_,1.),Mean);
    RUN;
    
    DATA _null_;
	set cov;
	call symput("omega"||put(_N_,1.)||"1",&varconst&actif1);
	call symput("omega"||put(_N_,1.)||"2",&varconst&actif2);
    RUN;

/* Si on est interesses au rendements non actualises */    
    %if &varconst=ypetit %then %do;
	%let d1=&&dernier&actif1;
	%let d2=&&dernier&actif2;
	%let mu1=%sysevalf(&d1 * (&mu1 - &tauxj));
	%let mu2=%sysevalf(&d2 * (&mu2 - &tauxj));
	%let omega11=%sysevalf(&omega11 * &d1 * &d1);
	%let omega12=%sysevalf(&omega12 * &d1 * &d2);
	%let omega21=%sysevalf(&omega21 * &d1 * &d2);
	%let omega22=%sysevalf(&omega22 * &d2 * &d2);
	%end;
    
/* Inversion de la matrice Omega */
    %let det=%sysevalf(&omega11* &omega22 - &omega12*&omega21);
    %let invdet=%sysevalf(1/&det);
    %let in11=%sysevalf(&omega22 * &invdet);
    %let in12=%sysevalf(-&omega12  *  &invdet);
    %let in21=%sysevalf(-&omega21  * &invdet);
    %let in22=%sysevalf(&omega11  * &invdet);
    
/* Calcul des  allocations optimales */
    %let alpha1= %sysevalf(1/&A*(&in11* &mu1 + &in12*&mu2));
    %let alpha2= %sysevalf(1/&A*(&in21* &mu1 + &in22*&mu2));
    %let alpha0= %sysevalf(&w - &alpha1*&&dernier&actif1 - &alpha2*&&dernier&actif2);

/* Calcul des la performance de Sharp */
    %let psharp=%sysevalf(&mu1*&mu1*&in11 + 2*&mu1*&mu2*&in12 + &mu2*&mu2*&in22);

/* Calcul de la valeur anticipe du portefeuille */    
    %let vaa=%sysevalf(&w * (1 + &tauxj) + 1/&A*&psharp);
  
    %mend allocation_performace;


/*** Macro calcul des donnes pour la couple d'actifs risques
    actif1 actif2 ***/

%macro exercice1(actif1,actif2);

    title "&actif1-&actif2";

 
    %lireactif(&actif1,&tauxj);
    %lireactif(&actif2,&tauxj);

    DATA tous&actif1&actif2 ;	
	merge &actif1 &actif2 ;	
    RUN;

    %if (&verbose=1) %then %do;
	PROC print data=tous&actif1&actif2;
	RUN;
	QUIT;
	%end;

    ods OUTPUT COVYlags=cov Summary=means;
    PROC varmax data=tous&actif1&actif2 ;
	model &varconst&actif1 &varconst&actif2
	    /p=1 print=(COVY) LAGMAX=0;
    RUN;
    QUIT;
    ODS OUTPUT CLOSE; 
    
    %if (&verbose=1) %then %do;
	proc print data=means;
	    title2 "Moyenne";
	RUN;
	QUIT;
	
	proc print data=cov;
	    title2 "Covariance";
	RUN;
	QUIT;
	%end;
    
    
    %let alpha0=0;
    %let alpha1=0;
    %let alpha2=0;
    %let psharp=0;
    %let vaa=0;
    %allocation_performace;
 
    data results&actif1&actif2;
	actif1="&actif1";
	actif2="&actif2";
	performance_sharp=&psharp;
	valeur_anticipe=&vaa;    
	montant=&w;
	mont_non_risque=&alpha0;
	no_actif1=&alpha1;
	prix_un_actif1=&&dernier&actif1;  
	mont_actif1=&&dernier&actif1*&alpha1;
	no_actif2=&alpha2;
	prix_un_actif2=&&dernier&actif2; 
	mont_actif2=&&dernier&actif2*&alpha2; 
    run;
    
    %if (&verbose=1) %then %do;
	proc print data=results&actif1&actif2;
	    title2 "Resultats pour &actif1-&actif2";
	run;
	quit;
	%end;
    
    %mend exercice1;

/* Macro : pour toute couple des actifs faire l'exercice 1 
   et comparer les resultats */
 
%macro exercicecomplet;
    
    %do i=1 %to %eval(&noactifs-1);
	%do j=%eval(&i+1) %to &noactifs;
	    %exercice1(&&act&i,&&act&j);
	    %end;
	%end;
    
    %let tous=;
    %do i=1 %to %eval(&noactifs-1);
	%do j=%eval(&i+1) %to &noactifs;
	    %let tous=&tous results&&act&i&&act&j;
	    %end;
	%end;
    
    DATA tous;
	set  &tous;
    Run; 
    
    proc sort data=tous;
	by valeur_anticipe;
    run;
    
    %mend exercicecomplet;
 

/****   PROGRAMME **********************************************/  
%exercicecomplet;

title "Tous les resultats ";

proc print data=tous;
run; 
    






