/* Luigi Santocanale */
/* Last modified : Mon Nov 17 23:49:56 CET 2003      */


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "syntaxe.h"
#include "formulesetmodeles.h"


/* Gestion de l'affichage */

void imprimer_formule(Formule *formule)
{
  
  if(formule == NULL)
    return;

  switch(formule->op)
    {
    case CONJ:
      printf("(");
      imprimer_formule(formule->filsagauche);
      printf(CONJ_CHAINE);
      imprimer_formule(formule->filsadroite);
      printf(")");
      return ;
    case DISJ:
      printf("(");
      imprimer_formule(formule->filsagauche);
      printf(DISJ_CHAINE);
      imprimer_formule(formule->filsadroite);
      printf(")");
      return ; 
    case IMPL:
      printf("(");
      imprimer_formule(formule->filsagauche);
      printf(IMPL_CHAINE);
      imprimer_formule(formule->filsadroite);
      printf(")");
      return ; 
    case NEG:
      printf(NEG_CHAINE);
      imprimer_formule(formule->filsagauche);
      return;
    case VAR:
      printf("%c", formule->var);
      return ;
    default:
      exit(EXIT_FAILURE);
    }
}

int est_satisfaite(Formule *formule,Modele *modele)
{
  /* Abort si on a pas une formule */ 
  assert(formule != NULL);

  switch(formule->op)
    {
    case CONJ:
      return 
	est_satisfaite(formule->filsagauche,modele)
	&&
	est_satisfaite(formule->filsadroite,modele);
    case DISJ:
      return 
	est_satisfaite(formule->filsagauche,modele)
	||
	est_satisfaite(formule->filsadroite,modele); 
    case IMPL:
      return 
	! est_satisfaite(formule->filsagauche,modele)
	||
	est_satisfaite(formule->filsadroite,modele);
    case NEG:
      return 
	! est_satisfaite(formule->filsagauche,modele);
    case VAR:
      return 
	modele->val[formule->var - FIRSTVAR];
    default:
      exit(EXIT_FAILURE);
    }
}

/*
  Construction des variables 
  a partir des chanes de caractres
*/

Variable strtovar(char *chaine)
{
  if(chaine == NULL)
    return FIRSTVAR;
  if( chaine[0] < FIRSTVAR || chaine[0] > LASTVAR )
    return FIRSTVAR;
  
  return (Variable) chaine[0]; 
}


/* Construction et destruction des formules */

Formule *conjonctionde(Formule *f1,Formule *f2)
{
  Formule *ret=malloc(sizeof(Formule));
  
  ret->op=CONJ;
  ret->filsagauche=f1;
  ret->filsadroite=f2;

  return ret;

}

Formule *disjonctionde(Formule *f1,Formule *f2)
{
  Formule *ret=malloc(sizeof(Formule));
  
  ret->op=DISJ;
  ret->filsagauche=f1;
  ret->filsadroite=f2;

  return ret;

}

Formule *implicationde(Formule *f1,Formule *f2)
{
  Formule *ret=malloc(sizeof(Formule));
  
  ret->op=IMPL;
  ret->filsagauche=f1;
  ret->filsadroite=f2;

  return ret;

}

Formule *negationde(Formule *f)
{
  Formule *ret=malloc(sizeof(Formule));
  
  ret->op=NEG;
  ret->filsagauche=f;
  ret->filsadroite=NULL;

  return ret;

}

Formule *variablede(Variable v)
{
  Formule *ret=malloc(sizeof(Formule));
  
  ret->op=VAR;
  ret->var=v;
  ret->filsagauche=NULL;
  ret->filsadroite=NULL;

  return ret;

}

void libererformule(Formule *f)
{
  if(f != NULL)
    {
      libererformule(f->filsagauche);
      libererformule(f->filsadroite);
      free(f);
    }  
}


/* Construction et destruction des modeles */

void liberermodele(Modele *m)
{
  if(m != NULL)
    {
      free(m);
    }  
}
